/*-------------------------------------------------------- 
 * Module Name : cattp and sms lib
 * Version : 1.7.1
 * 
 * Software Name : Transport Manager
 * Version : 1.7.1
 *
 * Copyright © 01-10-2012 – 01-10-2012 France Télécom
 * This software is distributed under the Apache (v2) license,
 * the text of which is available at http://www.apache.org/licenses/LICENSE-2.0.html
 * or see the "license.txt" file for more details.
 * 
 *--------------------------------------------------------
 * File Name   : ${file_name}
 *
 * Created     : ${date}
 * Author(s)   : ${authors}
 *
 * Description :
 * Main interface of this lib. It shall be called by the upper level
 *
 *--------------------------------------------------------
 * ${Log}
 *
 */
package com.orange.labs.quickpass.tm;

import com.orange.labs.quickpass.enums.TransportBearer;
import com.orange.labs.quickpass.tm.service.CattpSessionStatus;
import com.orange.labs.quickpass.tm.service.cattp.CattpConnectionInfo;
import com.orange.labs.quickpass.tm.service.cattp.NoMorePortAvailableException;
import com.orange.labs.quickpass.tm.service.cattp.SessionNotFoundException;
import com.orange.labs.quickpass.type.Msisdn;
import java.util.List;

/**
 * Transport Manager Interface Main interface of this lib. It shall be called by the upper level
 *
 * @author Gael Gerard and Sylvain Ducourval
 */
public interface TransportManager {

    /**
     * MAX_SMS_SIZE
     */
    int MAX_SMS_SIZE = 160;

    /**
     * Get a valid and available cattp port
     *
     * @param msisdn the msisdn
     * @return cattp information data when available, null otherwise.
     * @throws NoMorePortAvailableException
     */
    CattpConnectionInfo openCattpLink(Msisdn msisdn) throws NoMorePortAvailableException;

    /**
     * Sends an SMS text to the targeted msisdn
     *
     * @param msisdns targeted msisdns
     * @param message SMS text message to send
     */
    void sendSMSText(Msisdn[] msisdns, String message);

    /**
     * Sends a wap push SMS to the targeted MSISDN
     *
     * @param msisdns targeted msisdns
     * @param message message to send
     * @param url url to be pushed on browser
     */
    void sendWAPPushSMS(Msisdn[] msisdns, String message, String url);

    /**
     * Sends a Java Push Sms on a given port
     *
     * @param msisdns targeted msisdns
     * @param message associated message (may be null)
     * @param port port where SMS will be pushed.
     */
    void sendJavaPushSMS(Msisdn[] msisdns, String message, String port);

    /**
     * Sends OTA secure packets either using SMS or CATTP
     *
     * @param securePackets Secure packets array
     * @param msisdn targeted msisdn
     * @param bearer bearer (SMS or CATTP)
     * @throws SessionNotFoundException
     */
    void sendSecurePackets(Msisdn msisdn, List<byte[]> securePackets, TransportBearer bearer) throws SessionNotFoundException;

    /**
     * Closes cattp connection for a specific msisdn This function shall be explicitly called to call a cattp session
     *
     * @param msisdn the msisdn
     * @throws SessionNotFoundException
     */
    void closeCattpLink(Msisdn msisdn) throws SessionNotFoundException;

    /**
     * Get the status for a cattp session 
     * @param msisdn targeted msisdn
     * @return status {CREATED,OPENED,CLOSED}   
     * @throws SessionNotFoundException when no session has been found
     */
    CattpSessionStatus getCattpSessionStatus(Msisdn msisdn) throws SessionNotFoundException;
}
