/*-------------------------------------------------------- 
* Module Name : cattp and sms lib
* Version : 1.7.1
* 
* Software Name : Transport Manager
* Version : 1.7.1
*
* Copyright © 01-10-2012 – 01-10-2012 France Télécom
* This software is distributed under the Apache (v2) license,
* the text of which is available at http://www.apache.org/licenses/LICENSE-2.0.html
* or see the "license.txt" file for more details.
*--------------------------------------------------------
* File Name   : ${file_name}
*
* Created     : ${date}
* Author(s)   : ${authors}
*
* Description :
* Activity listener for a given session
*
*--------------------------------------------------------
* ${Log}
*
*/


package com.orange.labs.quickpass.tm.cattp;

import com.orange.labs.quickpass.tm.session.CATTPSession;

/**
 * CATTPActivityListener: CATTP activity listener interface
 * 
 */
public interface CATTPActivityListener {
    /**
     * Called when data has been received for the current session
     * 
     * @param session current cattp session
     */
    void onCattpDataReceived(CATTPSession session);

    /**
     * Called when the connection has been established for the current session
     * 
     * @param session  current cattp session
     */
    void onCattpConnectionEstablished(CATTPSession session);

    /**
     * Called when the current SDU has been successfully received (ACKed) by
     * remote entity
     * 
     * @param session  current cattp session
     */
    void onCattpSDUSuccessfullyReceived(CATTPSession session);

    /**
     * Called when no more PDU left to send
     * 
     * @param session  current cattp session
     */
    void onCattpNoMorePduForCurrentSduToSend(CATTPSession session);

    /**
     * Called when a status command has been responded by the remote entity
     * 
     * @param session  current cattp session
     */
    void onStatus(CATTPSession session);
}
