/*-------------------------------------------------------- 
* Module Name : cattp and sms lib
* Version : 1.7.1
* 
* Software Name : Transport Manager
* Version : 1.7.1
*
* Copyright © 01-10-2012 – 01-10-2012 France Télécom
* This software is distributed under the Apache (v2) license,
* the text of which is available at http://www.apache.org/licenses/LICENSE-2.0.html
* or see the "license.txt" file for more details.
*
*--------------------------------------------------------
* File Name   : ${file_name}
*
* Created     : ${date}
* Author(s)   : ${authors}
*
* Description :
* Object representing a retry for a given pdu.
*
*--------------------------------------------------------
* ${Log}
*
*/


package com.orange.labs.quickpass.tm.cattp.pdu;

import com.orange.labs.quickpass.tm.cattp.server.CATTPServer;
import com.orange.labs.quickpass.tm.cattp.states.CATTPCloseWait;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Object representing a retry for a given pdu.
 * 
 */
public class RetryPduTask implements Runnable {

    private static final Logger LOGGER = LoggerFactory.getLogger(RetryPduTask.class);
    CATTPPdu pdu;
    /**
     * nb retries
     */
    int currentRetryIdx = 0;
    /**
     * nb retries
     */
    int maxRetries = CATTPServer.getInstance().getRetriesCounter();

    /**
     * Builds a retry action for a given PDU, for a given retry nb
     *
     * @param pdu pdu
     * @param retryIdx retryIdx
     */
    public RetryPduTask(CATTPPdu pdu, int retryIdx) {
        this.pdu = pdu;
        this.currentRetryIdx = retryIdx;
    }

    @Override
    public void run() {
        LOGGER.debug("Retry action is about to start for " + (currentRetryIdx + 1) + " times for pdu "
                +pdu.getHeader().getSeq());
        if (currentRetryIdx < maxRetries) {
            LOGGER.debug("Retry n°" + (currentRetryIdx + 1) + " in progress for pdu : " + pdu.getHeader().getSeq()
                    + " and user " + pdu.getSession().getMsisdn());            
            pdu.sendAndScheduleNextRetry();            
            currentRetryIdx++;
            if (currentRetryIdx == maxRetries) {
                if (!pdu.isRSTSet()) {
                    LOGGER.warn("Last try for session / pdu ("
                            + Integer.toHexString(pdu.getSession().getCattpPort())
                            + " / " + pdu.getHeader().getSeq() + ")");
                } else {
                    LOGGER.info("Last Reset Pdu sent, connection should be closed");
                }
            }
        } else {            
            // notification d'expiration à la couche supérieure
            pdu.getSession().getTransportReply().notifyCatTpSessionExpiration(pdu.getSession().getMsisdn());
            pdu.getSession().setResetCause(RstPdu.MAX_RETRIES_EXCEEDED);
            if (!pdu.getSession().getState().equals(CATTPCloseWait.getInstance())) {
                pdu.getSession().setState(CATTPCloseWait.getInstance());
            }

            pdu.cancelRetry();
        }
    }
}
