/*-------------------------------------------------------- 
* Module Name : cattp and sms lib
* Version : 1.7.1
* 
* Software Name : Transport Manager
* Version : 1.7.1
*
* Copyright © 01-10-2012 – 01-10-2012 France Télécom
* This software is distributed under the Apache (v2) license,
* the text of which is available at http://www.apache.org/licenses/LICENSE-2.0.html
* or see the "license.txt" file for more details.
*
*--------------------------------------------------------
* File Name   : ${file_name}
*
* Created     : ${date}
* Author(s)   : ${authors}
*
* Description :
* The UDP server, handles udp datagrams
*
*--------------------------------------------------------
* ${Log}
*
*/

package com.orange.labs.quickpass.tm.cattp.server;

import com.orange.labs.quickpass.common.utils.ConvertUtils;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.*;
import java.util.Observable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;

/**
 * The UDP server, handles udp datagrams
 */
public abstract class AbstractUDPServer extends Observable implements Runnable {

    private static final Logger LOGGER = LoggerFactory.getLogger(AbstractUDPServer.class);
    /**
     * Udp listening port used, should be parameter
     */
    @Value("${udpport}")
    protected int port;
    /**
     * Size of inBuffer should be set to some max value
     */
    protected byte[] inBuffer = new byte[2000];
    /**
     * data
     */
    protected byte[] data = new byte[2000];
    /**
     * inMsg
     */
    protected DatagramPacket inMsg = null;
    /**
     * outMsg
     */
    protected DatagramPacket outMsg = null;
    /**
     * running
     */
    protected boolean running = false;
    /**
     * sock
     */
    protected DatagramSocket sock = null;
    /**
     * publicIpAddress
     */
    protected String publicIpAddress;
    /**
     * proxyHost
     */
    protected String proxyHost = null;
    /**
     * proxyPort
     */
    protected String proxyPort = null;
    /**
     * ipUrlResolver
     */
    @Value("${ipUrlResolver}")
    protected String ipUrlResolver;

    /**
     * @throws MalformedURLException
     * @throws IOException
     */
    protected void initPublicAccess() throws MalformedURLException, IOException {
        if (proxyHost != null) {
            System.setProperty("http.proxyHost", proxyHost);
        }
        if (proxyPort != null) {
            System.setProperty("http.proxyPort", proxyPort);
        }
        publicIpAddress = getPublicIpAdress();

    }

    private String getPublicIpAdress() throws MalformedURLException, IOException {
        // search public IP on targeted server
        URL autoIP = new URL(ipUrlResolver);
        BufferedReader in = new BufferedReader(new InputStreamReader(autoIP.openStream()));
        String line = in.readLine();
        return line.replaceAll("([a-zA-Z])*([<>/:])*", "").trim();
    }

    /**
     * Sends a udp datagram
     *
     * @param ipAdress ipAdress
     * @param udpPort the udpPort
     * @param datagramData the datagram data
     */
    public void sendDataGram(InetAddress ipAdress, int udpPort, byte[] datagramData) {
        outMsg = new DatagramPacket(datagramData.clone(), datagramData.length,
                ipAdress, udpPort);
        try {
            if (getSock() == null) {
                LOGGER.warn("No sending socket available..., "
                        + "socket not set. Message should be : "
                        + ConvertUtils.toHexString(datagramData, ":", 40));
            } else {
                getSock().send(outMsg);
            }
        } catch (Exception e) {
            LOGGER.error("Sending Datagram failure : " + e.getMessage());
        }
    }

    /**
     * The server still running ?
     *
     * @return true when server runs
     */
    public boolean isRunning() {
        return running;
    }

    /**
     *
     * @param sock Datagram socket
     */
    public void setSock(DatagramSocket sock) {
        this.sock = sock;
    }

    /**
     *
     * @return Datagram socket
     */
    public DatagramSocket getSock() {
        return sock;
    }

    /**
     *
     * @return UDP port
     */
    public int getPort() {
        return port;
    }

    /**
     *
     * @return public IP address
     */
    public String getPublicIpAddress() {
        return publicIpAddress;
    }

    /**
     *
     * @param proxyHost proxy host
     */
    public void setProxyHost(String proxyHost) {
        this.proxyHost = proxyHost;
    }

    /**
     *
     * @param proxyPort proxy port
     */
    public void setProxyPort(String proxyPort) {
        this.proxyPort = proxyPort;
    }
}
