/*-------------------------------------------------------- 
* Module Name : cattp and sms lib
* Version : 1.7.1
* 
* Software Name : Transport Manager
* Version : 1.7.1
*
* Copyright © 01-10-2012 – 01-10-2012 France Télécom
* This software is distributed under the Apache (v2) license,
* the text of which is available at http://www.apache.org/licenses/LICENSE-2.0.html
* or see the "license.txt" file for more details.
*
*--------------------------------------------------------
* File Name   : ${file_name}
*
* Created     : ${date}
* Author(s)   : ${authors}
*
* Description :
* The CLOSED state is valid when no connection exists between two ports
*
*--------------------------------------------------------
* ${Log}
*
*/

package com.orange.labs.quickpass.tm.cattp.states;

import com.orange.labs.quickpass.tm.cattp.server.CATTPServer;
import com.orange.labs.quickpass.tm.session.CATTPSession;

/**
 * The CLOSED state is valid when no connection exists between two ports.<BR/> 
 * If no connection exists at all, the CLOSED state PDU arrival events processing is optional.
 * 
 */
public final class CATTPClosed extends CATTPState {

    private static CATTPClosed instance = null;

    private CATTPClosed() {      
    }
    
    /**
     * @return the instance
     */
    public static CATTPClosed getInstance() {
        if (instance == null) {
            instance = new CATTPClosed();
        }
        return instance;
    }

    //{@inheritDoc} 
    @Override
    public synchronized void entry(CATTPSession session) {
        // cancel any pending retry
        session.getLastSentAckablePdu().cancelRetry();
        session.getSentPdu().cancelRetry();       
        // remove session
        CATTPServer.removeSession(session);
    }

    //{@inheritDoc} 
    @Override
    public String toString() {
        return " C L O S E D ";
    }

    //{@inheritDoc} 
    @Override
    public void exit(CATTPSession session) {

    }

    //{@inheritDoc} 
    @Override
    public void receiveSYN(CATTPSession session) {
        passiveOpenRequest(session);
    }

    private void passiveOpenRequest(CATTPSession session) {
        if (session.getReceivedPdu() == null) {
            return;
        }

        int iniSeqNb = session.getReceivedPdu().getHeader().getSeq();

        session.setRcvIniSeqNb(iniSeqNb);

        session.setMaxPduSize(session.getReceivedPdu().getMaxPduSize());
        session.setMaxSduSize(session.getReceivedPdu().getMaxSduSize());

        session.setIdent(session.getReceivedPdu().getIdent());
        // on passe les 2 �tats suite � suite dans
        // un passive open.
        session.setState(CATTPListen.getInstance());
    }

}
