/*-------------------------------------------------------- 
* Module Name : cattp and sms lib
* Version : 1.7.1
* 
* Software Name : Transport Manager
* Version : 1.7.1
*
* Copyright © 01-10-2012 – 01-10-2012 France Télécom
* This software is distributed under the Apache (v2) license,
* the text of which is available at http://www.apache.org/licenses/LICENSE-2.0.html
* or see the "license.txt" file for more details.
*
*--------------------------------------------------------
* File Name   : ${file_name}
*
* Created     : ${date}
* Author(s)   : ${authors}
*
* Description :
* The LISTEN state is entered after a passive Open request
*
*--------------------------------------------------------
* ${Log}
*
*/

package com.orange.labs.quickpass.tm.cattp.states;

import com.orange.labs.quickpass.tm.cattp.pdu.CATTPPdu;
import com.orange.labs.quickpass.tm.session.CATTPSession;

/** 
 * The LISTEN state is entered after a passive Open request. <BR/>
 * A connection record is created and the CAT_TP server waits
 * for an active connection request reception.
 */
public final class CATTPListen extends CATTPState {

    private static CATTPListen instance = null;

    private CATTPListen() {
    }

    /**
     * @return the instance
     */
    public static CATTPListen getInstance() {
        if (instance == null) {
            instance = new CATTPListen();
        }
        return instance;
    }

    //{@inheritDoc} 
    @Override
    public void entry(CATTPSession session) {
    }

    //{@inheritDoc} 
    @Override
    public String toString() {
        return " L I S T E N ";
    }
    
    //{@inheritDoc} 
    @Override
    public void exit(CATTPSession session) {
    }

    //{@inheritDoc} 
    @Override
    public synchronized void receiveSYN(CATTPSession session) {
        CATTPPdu receivedPdu = session.getReceivedPdu();
        if (receivedPdu == null) {
            return;
        }
        if (receivedPdu.isRSTSet()) {
            logger.warn(String.format("%s RST pdu received, Discarding PDU !", session));
            return;
        }
        if (receivedPdu.isACKSet()) {
            session.sendRST();
        }
        short maxPduSize = receivedPdu.getMaxPduSize();
        short maxSduSize = receivedPdu.getMaxSduSize();
        int winSize = receivedPdu.getHeader().getWinSize();
        logger.debug(String.format("%s Max Pdu Size : %s <BR/>Max Sdu Size : %s <BR/>Win Size : %s",
                session, maxPduSize, maxSduSize, winSize));
        session.setMaxPduSize(maxPduSize);
        session.setMaxSduSize(maxSduSize);
        session.setRcvIniWinSize(winSize);
        session.setLastInSequencePdu(receivedPdu);
        session.setState(CATTPSynRcvd.getInstance());
    }

    //{@inheritDoc} 
    @Override
    public void closeRequest(CATTPSession session) {
        session.setState(CATTPClosed.getInstance());
    }
}
