/*-------------------------------------------------------- 
* Module Name : cattp and sms lib
* Version : 1.7.1
* 
* Software Name : Transport Manager
* Version : 1.7.1
*
* Copyright © 01-10-2012 – 01-10-2012 France Télécom
* This software is distributed under the Apache (v2) license,
* the text of which is available at http://www.apache.org/licenses/LICENSE-2.0.html
* or see the "license.txt" file for more details.
*
*--------------------------------------------------------
* File Name   : ${file_name}
*
* Created     : ${date}
* Author(s)   : ${authors}
*
* Description :
* SYN-RCVD state is reached from the LISTEN state when a SYN PDU requesting a 
* connection is received from a CAT_TP client.
*
*--------------------------------------------------------
* ${Log}
*
*/

package com.orange.labs.quickpass.tm.cattp.states;

import com.orange.labs.quickpass.tm.session.CATTPSession;

/**
 * SYN-RCVD state is reached from the LISTEN state when a SYN PDU requesting a 
 * connection is received from a CAT_TP client. <BR/>In reply, the CAT_TP server shall 
 * generate an initial sequence number for its side of the connection,
 * and then sends the sequence number and an acknowledgement of the SYN PDU to 
 * the CAT_TP client. <BR/>It then waits for and acknowledgement.
 */
public final class CATTPSynRcvd extends CATTPState {

    private static CATTPSynRcvd instance = null;

    private CATTPSynRcvd() {
    }

    /**
     * @return the instance
     */
    public static CATTPSynRcvd getInstance() {
        if (instance == null) {
            instance = new CATTPSynRcvd();
        }
        return instance;
    }

    //{@inheritDoc} 
    @Override
    /**
     * Sequence number always = 100, this could be changed
     */
    public synchronized void entry(CATTPSession session) {
        session.sendSYNACK();            
    }

    //{@inheritDoc} 
    @Override
    public synchronized void exit(CATTPSession session) {
        session.provisionPackets();
    }

    //{@inheritDoc} 
    @Override
    public synchronized void receiveACK(CATTPSession session) {       
        if (session.getSentPdu().isSYNSet()) {           
            session.getSentPdu().cancelRetry();
        }        
        session.setState(CATTPOpen.getInstance());
    }

    //{@inheritDoc} 
    @Override
    public void receiveRST(CATTPSession session) {
        session.setState(CATTPClosed.getInstance());
    }

    //{@inheritDoc} 
    @Override
    public String toString() {
        return " S Y N - R C V D ";
    }
}
