/*-------------------------------------------------------- 
* Module Name : cattp and sms lib
* Version : 1.7.1
* 
* Software Name : Transport Manager
* Version : 1.7.1
*
* Copyright © 01-10-2012 – 01-10-2012 France Télécom
* This software is distributed under the Apache (v2) license,
* the text of which is available at http://www.apache.org/licenses/LICENSE-2.0.html
* or see the "license.txt" file for more details.
*
*--------------------------------------------------------
* File Name   : ${file_name}
*
* Created     : ${date}
* Author(s)   : ${authors}
*
* Description :
* CATTP Utils 
*
*--------------------------------------------------------
* ${Log}
*
*/

package com.orange.labs.quickpass.tm.cattp.utils;

import com.orange.labs.quickpass.common.utils.ConvertUtils;
import com.orange.labs.quickpass.tm.cattp.exceptions.BadCATTPChecksumException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * CATTP utils class, checksum
 */
public final class CATTPUtils {

    private static final Logger LOGGER = LoggerFactory.getLogger(CATTPUtils.class);
    
    private CATTPUtils() {
    }

    /**
     * checkSUM computation and verification
     * @param data data
     * @param check checkSUM
     * @return checksm
     * @throws BadCATTPChecksumException
     */
    public static byte[] checksum(byte[] data, boolean check)
            throws BadCATTPChecksumException {
        // TODO simplifier cette partie de code
        int donne1 = 0;
        int donne2 = 0;
        int checksum = 0;
        int somme = 0;

        boolean cks = false;

        // récupération du checksum meme si non existant cas du calcul
        // il faudrait essayer une autre methode
        // donne1=(int)(Data[16]&0x00FF);
        String foundChk = ConvertUtils.toHexString(data, "  ", 16, 2);

        // Supprimer superieur
        checksum = (data[16] & 0x00ff) + (data[17] << 8) & 0xffff;

        data[16] = 0;
        data[17] = 0;

        // calcul de la longueur header + datas
        int longueur = (data[9] & 0x00ff) + (data[8] << 8) & 0xffff;
        longueur += (data[3] & 0x00ff);

        int i = 0;
        while (longueur > 1) {
            if (data[i] < 0) {
                donne1 = data[i] + 256;
            } else {
                donne1 = data[i];
            }
            if (data[i + 1] < 0) {
                donne2 = data[i + 1] + 256;
            } else {
                donne2 = data[i + 1];
            }
            somme = (somme + donne2 * 256 + donne1);
            // limitation de la somme à 16 bits
            if (somme > 0xFFFF) {
                somme = somme & 0xFFFF;
                // si la somme dépasse 16 bits on ajoute 1 voir doc oberthur
                somme = somme + 1;
            }
            i++;
            i++;
            longueur = longueur - 2;
        }
        if (longueur == 1) {
            // traitement longueur :impaire
            if (data[i] < 0) {
                donne1 = data[i] + 256;
            } else {
                donne1 = data[i];
            }
            somme = somme + donne1;
            if (somme > 0xFFFF) {
                somme = somme & 0xFFFF;
                somme = somme + 1;
            }
        }
        // ou exclusif avec 0xFFFF = complément à 1
        somme = somme ^ 0xFFFF;
        if (somme == checksum) {
            cks = true;
        }
        // on replace le checksum pour les constructions
        data[16] = (byte) (somme % 256);
        data[17] = (byte) (somme / 256);

        if (!cks && check) {
            LOGGER.debug("Checksum is : " + foundChk + "  Should be : "
                    + ConvertUtils.toHexString(data, "  ", 16, 2));
            throw new BadCATTPChecksumException();
        }

        return new byte[] { data[16], data[17] };

    }
   
}
