/*-------------------------------------------------------- 
 * Module Name : cattp and sms lib
 * Version : 1.7.1
 * 
 * Software Name : Transport Manager
 * Version : 1.7.1
 *
 * Copyright © 01-10-2012 – 01-10-2012 France Télécom
 * This software is distributed under the Apache (v2) license,
 * the text of which is available at http://www.apache.org/licenses/LICENSE-2.0.html
 * or see the "license.txt" file for more details.
 *
 *--------------------------------------------------------
 * File Name   : ${file_name}
 *
 * Created     : ${date}
 * Author(s)   : ${authors}
 *
 * Description :
 * SMS Session 
 *
 *--------------------------------------------------------
 * ${Log}
 *
 */
package com.orange.labs.quickpass.tm.session;

import com.orange.labs.quickpass.tm.sms.gateway.SMSGateway;
import javax.annotation.Resource;

/**
 * SMSSession
 */
public class SMSSession extends AbstractTransportSession {

    @Resource
    private SMSGateway smsGateway;
    private int securePacketCursor;
    private String textMsg;
    private int pushPort;
    private byte[] binaryMessage;

    @Override
    public void run() {
        if (smsGateway != null && securePacketCursor < securePackets.size()) {
            binaryMessage = securePackets.get(securePacketCursor);
            smsGateway.sendBinarySMS(this);
        }
    }

    /**
     * @param sessionId sessionId
     */
    public void ack(String sessionId) {
        log.info("Ack received for : " + msisdn + " session Id : " + sessionId);
        this.sessionId = sessionId;
    }

    /**
     * nAck
     */
    public void nAck() {
        log.error("Sms NACK received for : " + msisdn);
    }

    //push port getter
    /**
     *
     * @return push Port
     */
    public int getPushPort() {
        return pushPort;
    }

    //push port setter
    /**
     *
     * @param pushPort push Port
     */
    public void setPushPort(int pushPort) {
        this.pushPort = pushPort;
    }

    /**
     *
     * @param securePacketCursor secured packet cursor setter
     */
    public void setSecurePacketCursor(int securePacketCursor) {
        this.securePacketCursor = securePacketCursor;
    }

    /**
     *
     * @return secured packet cursor getter
     */
    public int getSecurePacketCursor() {
        return securePacketCursor;
    }

    /**
     * increment the secure packet cursor
     */
    public void incrementSecurePacketCursor() {
        this.securePacketCursor++;
    }

    /**
     *
     * @return secured packet getter
     */
    public byte[] getBinaryMessage() {
        return binaryMessage.clone();
    }

    /**
     *
     * @return text msg getter
     */
    public String getTextMsg() {
        return textMsg;
    }

    /**
     *
     * @param textMsg text msg setter
     */
    public void setTextMsg(String textMsg) {
        this.textMsg = textMsg;
    }

    @Override
    public String toString() {
        return String.format("SmsSession[%s:packets size/cursor:(%s,%s)]: ",
                msisdn, securePackets.size(), securePacketCursor);
    }
}
