/*-------------------------------------------------------- 
 * Module Name : cattp and sms lib
 * Version : 1.7.1
 * 
 * Software Name : Transport Manager
 * Version : 1.7.1
 *
 * Copyright © 01-10-2012 – 01-10-2012 France Télécom
 * This software is distributed under the Apache (v2) license,
 * the text of which is available at http://www.apache.org/licenses/LICENSE-2.0.html
 * or see the "license.txt" file for more details.
 *
 *--------------------------------------------------------
 * File Name   : ${file_name}
 *
 * Created     : ${date}
 * Author(s)   : ${authors}
 *
 * Description :
 * SMS Gateway Implementation
 *
 *--------------------------------------------------------
 * ${Log}
 *
 */
package com.orange.labs.quickpass.tm.sms.gateway;

import com.orange.labs.quickpass.common.utils.ConvertUtils;
import com.orange.labs.quickpass.tm.gateway.SMSGatewayAdapter;
import com.orange.labs.quickpass.tm.session.SMSSession;
import java.util.StringTokenizer;
import javax.annotation.Resource;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * SMS Gateway Implementation. This classe works as a proxy (gatewayadapter)
 *
 */
public final class SMSGatewayImpl implements SMSGateway {

    private static final Logger LOGGER = LoggerFactory.getLogger(SMSGatewayImpl.class);
    /**
     * The adapter used for the Gateway, linked to actual SMS provider
     */
    @Resource
    private SMSGatewayAdapter smsGateWayAdapter;

    /**
     * Default constructor
     */
    private SMSGatewayImpl() {
    }

    //{@inheritDoc} 
    @Override
    public void sendText(SMSSession session) {
        smsGateWayAdapter.sendTextSMS(session.getMsisdn(), session.getTextMsg());
    }

    //{@inheritDoc} 
    @Override
    public void sendBinarySMS(SMSSession session) {
        byte[] binaryMessage = session.getBinaryMessage();
        LOGGER.debug(String.format("%s GateWay :%s, Sending binary SMS,  msg is : %s ",
                session, smsGateWayAdapter, ConvertUtils.toShortHexString(binaryMessage, ":", 4, 4)));
        smsGateWayAdapter.sendBinarySMS(session.getMsisdn(), binaryMessage);
    }

    //{@inheritDoc} 
    @Override
    public void sendPushSMS(SMSSession session) {
        int pushPort = session.getPushPort();
        LOGGER.debug(String.format("%s GateWay %s: Sending PUSH SMS on port : %s", session,
                smsGateWayAdapter, pushPort));
        smsGateWayAdapter.sendPushSMS(session.getMsisdn(), session.getTextMsg(), pushPort);
    }

    //{@inheritDoc} 
    @Override
    public void sendWapPushSMS(SMSSession session) {
        String textMsg = session.getTextMsg();
        LOGGER.debug(String.format("%s GateWay %s: , Sending WAP PUSH SMS: %s", session,
                smsGateWayAdapter, textMsg));
        StringTokenizer st = new StringTokenizer(textMsg, ";");
        String url = st.nextToken();
        String message = null;
        if (st.hasMoreTokens()) {
            message = st.nextToken();
        }
        smsGateWayAdapter.sendPushWAPSMS(session.getMsisdn(), url, message);
    }
}
