/*-------------------------------------------------------- 
* Module Name : cattp and sms lib
* Version : 1.7.1
* 
* Software Name : Transport Manager
* Version : 1.7.1
*
* Copyright © 01-10-2012 – 01-10-2012 France Télécom
* This software is distributed under the Apache (v2) license,
* the text of which is available at http://www.apache.org/licenses/LICENSE-2.0.html
* or see the "license.txt" file for more details.
*
*--------------------------------------------------------
* File Name   : ${file_name}
*
* Created     : ${date}
* Author(s)   : ${authors}
*
* Description :
* SMS Server Implementation (connected to the SMS broker through the gateway)
*
*--------------------------------------------------------
* ${Log}
*
*/

package com.orange.labs.quickpass.tm.sms.server;

import com.orange.labs.quickpass.tm.notifier.TransportReplyService;
import com.orange.labs.quickpass.tm.service.cattp.SessionNotFoundException;
import com.orange.labs.quickpass.tm.session.SMSSession;
import com.orange.labs.quickpass.type.Msisdn;
import java.util.HashMap;
import javax.annotation.Resource;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.task.TaskExecutor;

/**
 * SmsServer
 */
public class SMSServerImpl implements SMSServer {

    private static final Logger LOGGER = LoggerFactory.getLogger(SMSServerImpl.class);
    private static HashMap<String, SMSSession> storedSessions = new HashMap<String, SMSSession>();
    @Resource
    private TransportReplyService transportReply;
    @Resource
    private TaskExecutor taskExecutor;

    //{@inheritDoc} 
    @Override
    public void executeAndStoreSession(SMSSession session) {
        taskExecutor.execute(session);
        storedSessions.put(session.getMsisdn().getValue(), session);
    }

    //{@inheritDoc} 
    @Override
    public SMSSession getSessionByMsisdn(Msisdn msisdn) throws SessionNotFoundException {
        SMSSession session = storedSessions.get(msisdn.getValue());
        if (session == null) {
            throw new SessionNotFoundException("Session not found for " + msisdn);
        }
        return session;
    }

    //{@inheritDoc} 
    @Override
    public void onIncomingTextMessage(String msg) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    //{@inheritDoc} 
    @Override    
    public void onIncomingPor(Msisdn msisdn, byte[] por) {
        if (por != null) {
            transportReply.handleSEResponse(msisdn, por);
        }
    }

    //{@inheritDoc} 
    @Override   
    public void ack(String msisdn) {
    }
   

    //{@inheritDoc} 
    @Override   
    public void onSmsDelivered(Msisdn msisdn) {
        try {
            SMSSession smsSession = getSessionByMsisdn(msisdn);
            if (smsSession != null) {
                LOGGER.debug(String.format("%s Next SMS will be send...", smsSession));
                smsSession.incrementSecurePacketCursor();
                executeAndStoreSession(smsSession);
            }
        } catch (SessionNotFoundException noSession) {
            LOGGER.warn(String.format("[{%s}] - No session context found for this msisdn", msisdn));
        }
    }
}
